<?php

namespace Moblyze\Site\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Moblyze\Site\Http\Requests\SiteRequest;
use Moblyze\Site\Interfaces\SiteRepositoryInterface;
use Moblyze\Site\Models\Site;

/**
 * Resource controller class for site.
 */
class SiteResourceController extends BaseController
{

    /**
     * Initialize site resource controller.
     *
     * @param type SiteRepositoryInterface $site
     *
     * @return null
     */
    public function __construct(SiteRepositoryInterface $site)
    {
        parent::__construct();
        $this->repository = $site;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Moblyze\Site\Repositories\Criteria\SiteResourceCriteria::class);
    }

    /**
     * Display a list of site.
     *
     * @return Response
     */
    public function index(SiteRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Moblyze\Site\Repositories\Presenter\SitePresenter::class)
                ->$function();
        }

        $sites = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('site::site.names'))
            ->view('site::site.index', true)
            ->data(compact('sites', 'view'))
            ->output();
    }

    /**
     * Display site.
     *
     * @param Request $request
     * @param Model   $site
     *
     * @return Response
     */
    public function show(SiteRequest $request, Site $site)
    {

        if ($site->exists) {
            $view = 'site::site.show';
        } else {
            $view = 'site::site.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('site::site.name'))
            ->data(compact('site'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new site.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SiteRequest $request)
    {

        $site = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('site::site.name')) 
            ->view('site::site.create', true) 
            ->data(compact('site'))
            ->output();
    }

    /**
     * Create new site.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SiteRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $site                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('site::site.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('site/site/' . $site->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/site/site'))
                ->redirect();
        }

    }

    /**
     * Show site for editing.
     *
     * @param Request $request
     * @param Model   $site
     *
     * @return Response
     */
    public function edit(SiteRequest $request, Site $site)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('site::site.name'))
            ->view('site::site.edit', true)
            ->data(compact('site'))
            ->output();
    }

    /**
     * Update the site.
     *
     * @param Request $request
     * @param Model   $site
     *
     * @return Response
     */
    public function update(SiteRequest $request, Site $site)
    {
        try {
            $attributes = $request->all();

            $site->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('site::site.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('site/site/' . $site->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('site/site/' . $site->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the site.
     *
     * @param Model   $site
     *
     * @return Response
     */
    public function destroy(SiteRequest $request, Site $site)
    {
        try {

            $site->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('site::site.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('site/site/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('site/site/' . $site->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple site.
     *
     * @param Model   $site
     *
     * @return Response
     */
    public function delete(SiteRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('site::site.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('site/site'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/site/site'))
                ->redirect();
        }

    }

    /**
     * Restore deleted sites.
     *
     * @param Model   $site
     *
     * @return Response
     */
    public function restore(SiteRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('site::site.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/site/site'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/site/site/'))
                ->redirect();
        }

    }

}
